# 对象接口

使用接口，可以指定某个类必须实现哪些方法，但不需要定义这些方法的具体内容。接口是通过 `interface` 关键字来定义的，就像定义一个标准的类一样，但其中定义所有的方法都是空的。接口中定义的所有方法都必须是公有，这是接口的特性。要实现一个接口，使用 `implements` 操作符。类中必须实现接口中定义的所有方法，必须使用和接口中所定义的方法完全一致的方式，否则会报一个致命错误。

```php
<?php

interface Foo
{
    /**
     * Just an abstract method.
     *
     * @param  string $string
     * @return string
     */
    public function method(string $string): string;
}

class Bar implements Foo
{
    /**
     * Override the abstract method.
     *
     * @param  string $string
     * @return string
     */
    public function method(string $string): string
    {
        return $string;
    }
}

class Baz implements Foo
{
    /**
     * Override the abstract method.
     *
     * @param  string    $string
     * @param  bool|null $option
     * @return string
     */
    public function method(string $string, ?bool $option = null): string
    {
        return $string;
    }
}

$bar = new Bar();
var_dump($bar->method('bar')); // string(3) "bar"

$baz = new Baz();
var_dump($baz->method('baz')); // string(3) "baz"

```

接口也可以继承，通过使用 `extends` 操作符。

```php
<?php

interface Foo
{
    /**
     * Just an abstract method.
     *
     * @param  void
     * @return void
     */
    public function foo();
}

interface Bar extends Foo
{
    /**
     * Just an abstract method.
     *
     * @param  void
     * @return void
     */
    public function bar();
}

class Baz implements Bar
{
    /**
     * Override the abstract method.
     *
     * @param  void
     * @return void
     */
    public function foo()
    {
        //
    }

    /**
     * Override the abstract method.
     *
     * @param  void
     * @return void
     */
    public function bar()
    {
        //
    }
}

```

类可以实现多个接口，用逗号来分隔多个接口的名称。实现多个接口时，接口中的方法不能有重名。

```php
<?php

interface Foo
{
    /**
     * Just an abstract method.
     *
     * @param  void
     * @return void
     */
    public function foo();
}

interface Bar
{
    /**
     * Just an abstract method.
     *
     * @param  void
     * @return void
     */
    public function bar();
}

interface Baz
{
    /**
     * Just an abstract method.
     *
     * @param  void
     * @return void
     */
    public function baz();
}

class Qux implements Foo, Bar, Baz
{
    /**
     * Override the abstract method.
     *
     * @param  void
     * @return void
     */
    public function foo()
    {
        //
    }

    /**
     * Override the abstract method.
     *
     * @param  void
     * @return void
     */
    public function bar()
    {
        //
    }

    /**
     * Override the abstract method.
     *
     * @param  void
     * @return void
     */
    public function baz()
    {
        //
    }
}

```

接口中也可以定义常量。接口中的常量和类常量的使用完全相同，但是不能被子类或子接口所重写。

```php
<?php

interface Foo
{
    /**
     * Just a test constant.
     *
     * @var string
     */
    public const CONSTANT = 'foo';
}

interface Bar extends Foo
{
    //
}

class Baz implements Bar
{
    //
}

var_dump(Foo::CONSTANT); // string(3) "foo"
var_dump(Bar::CONSTANT); // string(3) "foo"
var_dump(Baz::CONSTANT); // string(3) "foo"

```

